!******************************************************************************!
!                                                                              !
!    class_vertices.f90                                                        !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   class_vertices.f90                             |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 2015                                 |            !
!        |       | UFO version: 0.1                               |            !
!        |       | Generation date: 2020-07-29 12:50             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module class_vertices
  use class_particles, only: n_orders
  use constants_mdl
  use dictionary, only: dict_init
  implicit none
  save

  type Branch
    integer :: lorentz_id
    integer :: perm
    integer, allocatable :: couplings(:)
    integer, dimension(n_orders) :: coupling_order
    integer :: color_id
    integer :: colorfac_id
    ! Propagator extension (only supported for massive vectors)
    ! 0: Usual propagator
    ! 1: Rxi-propagator part with pole @ M_V^2
    ! 2: Rxi-propagator part with pole @ \xi*M_V^2
    integer :: pext
  end type

  type Vertex
    type(Branch), allocatable :: branch(:)
    type(Branch), allocatable :: branchCT(:)
    type(Branch), allocatable :: branchR2(:)
  end type

  logical :: vertices_init = .false.
  logical :: vertices_filled = .false.
  integer, parameter :: n_vertices = 1208
  type(Vertex) :: vertices(0:n_vertices-1)

  ! Defines the maximum allowed number of legs participating at a vertex
  integer, parameter :: max_leg_multiplicity = 4

  integer, parameter :: VVV0 = 1
  integer, parameter :: VVVV0 = 2
  integer, parameter :: UaUV0 = 3
  integer, parameter :: UaUS0 = 4
  integer, parameter :: SSSS0 = 5
  integer, parameter :: SSS0 = 6
  integer, parameter :: VVSS0 = 7
  integer, parameter :: VSS0 = 8
  integer, parameter :: VVS0 = 9
  integer, parameter :: FFaV0 = 10
  integer, parameter :: FFaV1 = 11
  integer, parameter :: FFaS0 = 12
  integer, parameter :: FFaS1 = 13
  integer, parameter :: FFaS2 = 14
  integer, parameter :: VUaU0 = 15
  integer, parameter :: VUUa0 = 16
  integer, parameter :: UaVU0 = 17
  integer, parameter :: UUaV0 = 18
  integer, parameter :: UVUa0 = 19
  integer, parameter :: UaSU0 = 20
  integer, parameter :: UUaS0 = 21
  integer, parameter :: USUa0 = 22
  integer, parameter :: SUaU0 = 23
  integer, parameter :: SUUa0 = 24
  integer, parameter :: VSVS0 = 25
  integer, parameter :: VSSV0 = 26
  integer, parameter :: SVVS0 = 27
  integer, parameter :: SVSV0 = 28
  integer, parameter :: SSVV0 = 29
  integer, parameter :: SVS0 = 30
  integer, parameter :: SSV0 = 31
  integer, parameter :: VSV0 = 32
  integer, parameter :: SVV0 = 33
  integer, parameter :: FVFa0 = 34
  integer, parameter :: FVFa1 = 35
  integer, parameter :: VFFa0 = 36
  integer, parameter :: VFFa1 = 37
  integer, parameter :: VFaF0 = 38
  integer, parameter :: VFaF1 = 39
  integer, parameter :: FaFV0 = 40
  integer, parameter :: FaFV1 = 41
  integer, parameter :: FaVF0 = 42
  integer, parameter :: FaVF1 = 43
  integer, parameter :: SFFa0 = 44
  integer, parameter :: SFFa1 = 45
  integer, parameter :: SFFa2 = 46
  integer, parameter :: SFaF0 = 47
  integer, parameter :: SFaF1 = 48
  integer, parameter :: SFaF2 = 49
  integer, parameter :: FSFa0 = 50
  integer, parameter :: FSFa1 = 51
  integer, parameter :: FSFa2 = 52
  integer, parameter :: FaFS0 = 53
  integer, parameter :: FaFS1 = 54
  integer, parameter :: FaFS2 = 55
  integer, parameter :: FaSF0 = 56
  integer, parameter :: FaSF1 = 57
  integer, parameter :: FaSF2 = 58
  integer, parameter :: SS0 = 59
  integer, parameter :: SS1 = 60
  integer, parameter :: SS2 = 61
  integer, parameter :: FFa0 = 62
  integer, parameter :: FFa1 = 63
  integer, parameter :: FFa2 = 64
  integer, parameter :: FFa3 = 65
  integer, parameter :: VV0 = 66
  integer, parameter :: VV1 = 67
  integer, parameter :: VV2 = 68
  integer, parameter :: VV3 = 69
  integer, parameter :: VS0 = 70
  integer, parameter :: FaF0 = 71
  integer, parameter :: FaF1 = 72
  integer, parameter :: FaF2 = 73
  integer, parameter :: FaF3 = 74
  integer, parameter :: SV0 = 75
  complex(dp), parameter :: c1 = cmplx(0d0, -sq2/2d0, kind=dp)
  character(len=20), parameter :: c1f = "-i_*sq2/2"
  complex(dp), parameter :: c2 = cmplx(0d0, sq2/2d0, kind=dp)
  character(len=20), parameter :: c2f = "i_*sq2/2"
  complex(dp), parameter :: c3 = cmplx(-1d0/2d0, 0d0, kind=dp)
  character(len=20), parameter :: c3f = "-1/2"
  complex(dp), parameter :: c4 = cmplx(1d0/2d0, 0d0, kind=dp)
  character(len=20), parameter :: c4f = "1/2"
  complex(dp), parameter :: c5 = cmplx(1d0, 0d0, kind=dp)
  character(len=20), parameter :: c5f = "1"
  complex(dp), parameter :: c6 = cmplx(sq2/2d0, 0d0, kind=dp)
  character(len=20), parameter :: c6f = "sq2/2"
  complex(dp), parameter :: c7 = cmplx(-sq2/(2d0*Nc), 0d0, kind=dp)
  character(len=20), parameter :: c7f = "-sq2/(2*Nc)"
  complex(dp), parameter :: c8 = cmplx(-1d0/Nc, 0d0, kind=dp)
  character(len=20), parameter :: c8f = "-1/Nc"
  complex(dp), dimension(1:8) :: colorcoeffs = (/c1, c2, c3, c4, c5, c6, c7, c8&
                                               /)
  character(len=20), dimension(1:8) :: colorcoeffs_form = (/c1f, c2f, c3f, c4f &
                 , c5f, c6f, c7f, c8f/)
  integer, dimension(1:75) :: lorentz_rank = (/1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0 &
                                             , 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0 &
                                             , 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1 &
                                             , 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0 &
                                             , 1, 1, 1, -1, -1, -1, -1, -1, -1 &
                                             , -1, -1, -1, -1, -1, -1, -1, -1  &
                                             , -1, -1, -1/)
  integer, dimension(1:75) :: ff_ids = (/0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                       , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                       , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                       , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                       , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                       , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0/)
  integer, dimension(1:75) :: recola_base = (/12, -11, 164, 1, -1, 1, -4, 4, 2 &
                                            , 15, 17, 5, 7, 9, 162, 163, 166,  &
                                            161, 165, 1, 1, 1, 1, 1, -3, -13, -&
                                            2, -14, -12, 3, 11, 14, 13, 31, 35 &
                                            , 33, 36, 53, 58, 16, 18, 51, 57,  &
                                            23, 26, 28, 43, 46, 48, 21, 25, 27 &
                                            , 6, 8, 10, 41, 45, 47, 0, 0, 0, 0 &
                                            , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, &
                                            0, 0/)
  integer, dimension(1:75) :: propagator_extension = (/0, 0, 0, 0, 0, 0, 0, 0, &
                                                     0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                                     , 0, 0, 0, 0, 0, 0, 0, 0, &
                                                     0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                                     , 0, 0, 0, 0, 0, 0, 0, 0, &
                                                     0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                                     , 0, 0, 0, 0, 0, 0, 0, 0, &
                                                     0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                                     , 0, 0, 0, 0, 0, 0, 0/)
  character(len=*), parameter :: model_timestamp = "Wed Jul 29 12:34:19 2020"

  contains
    function get_timestamp_mdl() result(ret_timestamp)
      character(len=len(model_timestamp)) :: ret_timestamp
      ret_timestamp = model_timestamp
    end function get_timestamp_mdl

    function get_lorentz_rank_mdl(l_id) result(ret_lorentz_rank)
      integer, intent(in) :: l_id
      integer :: ret_lorentz_rank
      ret_lorentz_rank = lorentz_rank(l_id)
    end function get_lorentz_rank_mdl

    function get_propagator_extension_mdl(l_id) result(ret_pext)
      integer, intent(in) :: l_id
      integer :: ret_pext
      ret_pext = propagator_extension(l_id)
    end function get_propagator_extension_mdl

    function get_max_leg_multiplicity_mdl() result(ret_max_leg_multiplicity)
      integer :: ret_max_leg_multiplicity
      ret_max_leg_multiplicity = max_leg_multiplicity
    end function get_max_leg_multiplicity_mdl

    function get_fourfermion_id_mdl(l_id) result(ret_ffid)
      integer, intent(in) :: l_id
      integer :: ret_ffid
      ret_ffid = ff_ids(l_id)
    end function get_fourfermion_id_mdl

    function is_model_init_mdl() result(ret_init)
      logical :: ret_init
      ret_init = vertices_init
    end function is_model_init_mdl

    function is_model_filled_mdl() result(ret_init)
      logical :: ret_init
      ret_init = vertices_filled
    end function is_model_filled_mdl

    subroutine createCurrents
      use fill_currents, only: fill_currents_mdl
      implicit none
      call fill_currents_mdl(.false.)
    end subroutine createCurrents

    subroutine check_vertices_mdl
      use fill_currents, only: fill_currents_mdl
      implicit none
      call fill_currents_mdl(.true.)
    end subroutine check_vertices_mdl

    subroutine av(vid,bid,cid,cfid,pext,lid,perm,co,cop)
      integer, intent(in) :: vid,bid,cid,cfid,pext,lid,perm
      integer, dimension(0:), intent(in) :: co, cop
      integer scop,i
      vertices(vid)%branch(bid)%color_id = cid
      vertices(vid)%branch(bid)%colorfac_id = cfid
      vertices(vid)%branch(bid)%pext = pext
      vertices(vid)%branch(bid)%lorentz_id = lid
      vertices(vid)%branch(bid)%perm = perm
      vertices(vid)%branch(bid)%coupling_order = co
      scop = size(cop)
      allocate(vertices(vid)%branch(bid)%couplings(0:scop-1))
      do i = 0, scop-1
        vertices(vid)%branch(bid)%couplings(i) = cop(i)
      end do
    end subroutine av

    subroutine avCT(vid,bid,cid,cfid,pext,lid,perm,co,cop)
      integer, intent(in) :: vid,bid,cid,cfid,pext,lid,perm
      integer, dimension(0:), intent(in) :: co, cop
      integer scop,i
      vertices(vid)%branchCT(bid)%color_id = cid
      vertices(vid)%branchCT(bid)%colorfac_id = cfid
      vertices(vid)%branchCT(bid)%pext = pext
      vertices(vid)%branchCT(bid)%lorentz_id = lid
      vertices(vid)%branchCT(bid)%perm = perm
      vertices(vid)%branchCT(bid)%coupling_order = co
      scop = size(cop)
      allocate(vertices(vid)%branchCT(bid)%couplings(0:scop-1))
      do i = 0, scop-1
        vertices(vid)%branchCT(bid)%couplings(i) = cop(i)
      end do
    end subroutine avCT

    subroutine avR2(vid,bid,cid,cfid,pext,lid,perm,co,cop)
      integer, intent(in) :: vid,bid,cid,cfid,pext,lid,perm
      integer, dimension(0:), intent(in) :: co, cop
      integer scop,i
      vertices(vid)%branchR2(bid)%color_id = cid
      vertices(vid)%branchR2(bid)%colorfac_id = cfid
      vertices(vid)%branchR2(bid)%pext = pext
      vertices(vid)%branchR2(bid)%lorentz_id = lid
      vertices(vid)%branchR2(bid)%perm = perm
      vertices(vid)%branchR2(bid)%coupling_order = co
      scop = size(cop)
      allocate(vertices(vid)%branchR2(bid)%couplings(0:scop-1))
      do i = 0, scop-1
        vertices(vid)%branchR2(bid)%couplings(i) = cop(i)
      end do
    end subroutine avR2

    function get_colourfac_mdl(cfid)
      integer, intent(in) :: cfid
      complex(dp) :: get_colourfac_mdl
      get_colourfac_mdl = colorcoeffs(cfid)
    end function get_colourfac_mdl

    function get_colourfac_form_mdl(cfid)
      integer, intent(in) :: cfid
      character(len=20) :: get_colourfac_form_mdl
      get_colourfac_form_mdl = colorcoeffs_form(cfid)
    end function get_colourfac_form_mdl

  function get_recola_base_mdl(l_id) result(ret_rb)
      integer, intent(in) :: l_id
      integer :: ret_rb
      ret_rb = recola_base(l_id)
    end function get_recola_base_mdl

  subroutine get_vertex_branch_status_mdl(vx,xlp,branch_exists)
    integer, intent(in)  :: vx,xlp
    logical, intent(out) ::branch_exists
    select case (xlp)
    case(TreeBranch:LoopBranch)
      branch_exists = allocated(vertices(vx)%branch)
    case(CTBranch)
      branch_exists = allocated(vertices(vx)%branchCT)
    case(R2Branch)
      branch_exists = allocated(vertices(vx)%branchR2)
    end select
  end subroutine get_vertex_branch_status_mdl

  subroutine dealloc_vertex_mdl(vx,xlp)
    integer, intent(in)  :: vx,xlp
    logical ::branch_exists
    if (vx .ge. 0 .and.  vx .lt. n_vertices) then
      select case (xlp)
      case(TreeBranch:LoopBranch)
        if (allocated(vertices(vx)%branch)) deallocate(vertices(vx)%branch)
      case(CTBranch)
        if (allocated(vertices(vx)%branchCT)) deallocate(vertices(vx)%branchCT)
      case(R2Branch)
        if (allocated(vertices(vx)%branchR2)) deallocate(vertices(vx)%branchR2)
      case default
        if (allocated(vertices(vx)%branch)) deallocate(vertices(vx)%branch)
        if (allocated(vertices(vx)%branchCT)) deallocate(vertices(vx)%branchCT)
        if (allocated(vertices(vx)%branchR2)) deallocate(vertices(vx)%branchR2)
      end select
    end if
  end subroutine dealloc_vertex_mdl

  subroutine get_vertex_colour_branches_mdl(vx,xlp,cfids,nbranches)
    integer, allocatable, intent(out) :: cfids(:)
    integer, intent(out) :: nbranches
    integer, intent(in)  :: vx,xlp
    select case (xlp)
    case (TreeBranch:LoopBranch)
      allocate(cfids(size(vertices(vx)%branch)))
      cfids(:) = vertices(vx)%branch(0:)%color_id
      nbranches = size(vertices(vx)%branch)
    case(CTBranch)
      allocate(cfids(size(vertices(vx)%branchCT)))
      cfids(:) = vertices(vx)%branchCT(0:)%color_id
      nbranches = size(vertices(vx)%branchCT)
    case(R2Branch)
      allocate(cfids(size(vertices(vx)%branchR2)))
      cfids(:) = vertices(vx)%branchR2(0:)%color_id
      nbranches = size(vertices(vx)%branchR2)
    end select
  end subroutine get_vertex_colour_branches_mdl

  subroutine get_vertex_pext_mdl(vx,xlp,br,pext)
    integer, intent(in) :: vx,xlp,br
    integer, intent(out) :: pext
    select case (xlp)
    case (TreeBranch:LoopBranch)
      pext = vertices(vx)%branch(br)%pext
    case(CTBranch)
      pext = vertices(vx)%branchCT(br)%pext
    case(R2Branch)
      pext = vertices(vx)%branchR2(br)%pext
    end select
  end subroutine get_vertex_pext_mdl

  subroutine get_vertex_order_increase_mdl(vx,xlp,br,orderInc)
    integer, intent(in) :: vx,xlp,br
    integer, dimension(:), intent(out) :: orderInc
    select case (xlp)
    case (TreeBranch:LoopBranch)
      orderInc = vertices(vx)%branch(br)%coupling_order
    case(CTBranch)
      orderInc = vertices(vx)%branchCT(br)%coupling_order
    case(R2Branch)
      orderInc = vertices(vx)%branchR2(br)%coupling_order
    end select
  end subroutine get_vertex_order_increase_mdl

  subroutine get_vertex_lorentz_type_mdl(vx,xlp,br,lid)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: lid
    select case (xlp)
    case (TreeBranch:LoopBranch)
      lid = vertices(vx)%branch(br)%lorentz_id
    case(CTBranch)
      lid = vertices(vx)%branchCT(br)%lorentz_id
    case(R2Branch)
      lid = vertices(vx)%branchR2(br)%lorentz_id
    end select
  end subroutine get_vertex_lorentz_type_mdl

  subroutine get_vertex_perm_mdl(vx,xlp,br,perm)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: perm
    select case (xlp)
    case (TreeBranch:LoopBranch)
      perm = vertices(vx)%branch(br)%perm
    case(CTBranch)
      perm = vertices(vx)%branchCT(br)%perm
    case(R2Branch)
      perm = vertices(vx)%branchR2(br)%perm
    end select
  end subroutine get_vertex_perm_mdl

  subroutine get_vertex_colour_id_mdl(vx,xlp,br,cid)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: cid
    select case (xlp)
    case (TreeBranch:LoopBranch)
      cid = vertices(vx)%branch(br)%colorfac_id
    case(CTBranch)
      cid = vertices(vx)%branchCT(br)%colorfac_id
    case(R2Branch)
      cid = vertices(vx)%branchR2(br)%colorfac_id
    end select
  end subroutine get_vertex_colour_id_mdl

  function is_zero_coupling_vertex_mdl(vx,xlp,br) result(zero)
    use class_couplings, only: get_coupling_value_mdl
    use input_mdl, only: zerocut
    integer, intent(in)  :: vx,xlp,br
    logical              :: zero
    real(dp)             :: maxvalue
    integer              :: i

    zero = .false.
    select case (xlp)
    case (TreeBranch:LoopBranch)
      maxvalue = 0d0
      do i = 1,size(vertices(vx)%branch(br)%couplings)
        maxvalue = maxvalue + &
                abs(get_coupling_value_mdl(vertices(vx)%branch(br)%couplings(i-&
                1)))
      end do
      if (maxvalue/zerocut .lt. 100d0) then
        zero = .true.
      end if
    case(CTBranch)
      maxvalue = 0d0
      do i = 1,size(vertices(vx)%branchCT(br)%couplings)
        maxvalue = maxvalue + &
                abs(get_coupling_value_mdl(vertices(vx)%branchCT(br)%couplings(&
                i-1)))
      end do
      if (maxvalue/zerocut .lt. 100d0) then
        zero = .true.
      end if
    case(R2Branch)
      maxvalue = 0d0
      do i = 1,size(vertices(vx)%branchR2(br)%couplings)
        maxvalue = maxvalue + &
                abs(get_coupling_value_mdl(vertices(vx)%branchR2(br)%couplings(&
                i-1)))
      end do
      if (maxvalue/zerocut .lt. 100d0) then
        zero = .true.
      end if
    end select
  end function is_zero_coupling_vertex_mdl

  subroutine get_vertex_ncouplings_mdl(vx,xlp,br,ncouplings)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: ncouplings
    select case(xlp)
    case(TreeBranch, LoopBranch)
      ncouplings = size(vertices(vx)%branch(br)%couplings)
    case (CTBranch)
      ncouplings = size(vertices(vx)%branchCT(br)%couplings)
    case (R2Branch)
      ncouplings = size(vertices(vx)%branchR2(br)%couplings)
    end select
  end subroutine get_vertex_ncouplings_mdl

  subroutine get_vertex_coupling_ids_mdl(vx,xlp,br,cids)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: cids(:)
    select case(xlp)
    case(TreeBranch, LoopBranch)
      cids(:) = vertices(vx)%branch(br)%couplings(:)
    case (CTBranch)
      cids(:) = vertices(vx)%branchCT(br)%couplings(:)
    case (R2Branch)
      cids(:) = vertices(vx)%branchR2(br)%couplings(:)
    end select
  end subroutine get_vertex_coupling_ids_mdl

  subroutine init_vertices_mdl()
    use input_mdl, only: nx, error_mdl
    use class_particles,only: n_orders,driver_timestamp
    use fill_ctparameters, only: init_CTparameters_mdl
    use fill_couplings,only: fill_couplings_mdl,fill_CTcouplings_mdl

    ! check if modelfile is compatible to drivers
    if (model_timestamp .ne. driver_timestamp) then
      call error_mdl("Model corrupted. Driver timestamp mismatch.", &
                    where='init_vertices')
    end if

    vertices_init = .true.
    call fill_couplings_mdl
    call init_CTparameters_mdl(.true.)
    call fill_CTcouplings_mdl
    call dict_init(2417)
    call createCurrents

  end subroutine init_vertices_mdl

    subroutine get_vertex_lorentz_rank_mdl(ty,cc,mr)
      integer, intent(in) :: ty
      logical, intent(in) :: cc(0:)
      integer, intent(out) :: mr
      select case(ty)
      case(1)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(3)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(8)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(15)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(16)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(17)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(18)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(19)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(30)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(31)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(34)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(35)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(36)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(37)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(38)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(39)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(42)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(43)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(44)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(45)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(46)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(47)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(48)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(49)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(50)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(51)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(52)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(56)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(57)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(58)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case default
        mr = 0
      end select 
    end subroutine get_vertex_lorentz_rank_mdl
end module class_vertices
